import { dataSource } from "../../../data-source";
import { communication } from "../../../entity/club/member/communication";
import { member } from "../../../entity/club/member/member";
import InternalException from "../../../exceptions/internalException";
import { CreateMemberCommand, DeleteMemberCommand, UpdateMemberCommand } from "./memberCommand";

export default abstract class MemberCommandHandler {
  /**
   * @description create member
   * @param {CreateMemberCommand} createMember
   * @returns {Promise<number>}
   */
  static async create(createMember: CreateMemberCommand): Promise<number> {
    return await dataSource
      .createQueryBuilder()
      .insert()
      .into(member)
      .values({
        salutationId: createMember.salutationId,
        firstname: createMember.firstname,
        lastname: createMember.lastname,
        nameaffix: createMember.nameaffix,
        birthdate: createMember.birthdate,
        internalId: createMember.internalId,
      })
      .execute()
      .then((result) => {
        return result.identifiers[0].id;
      })
      .catch((err) => {
        throw new InternalException(
          `Failed creating member${err.code.includes("ER_DUP_ENTRY") ? " due to duplicate entry for column" : ""}`,
          err
        );
      });
  }

  /**
   * @description update member
   * @param {UpdateMemberCommand} updateMember
   * @returns {Promise<void>}
   */
  static async update(updateMember: UpdateMemberCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .update(member)
      .set({
        salutationId: updateMember.salutationId,
        firstname: updateMember.firstname,
        lastname: updateMember.lastname,
        nameaffix: updateMember.nameaffix,
        birthdate: updateMember.birthdate,
        internalId: updateMember.internalId,
      })
      .where("id = :id", { id: updateMember.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new InternalException(
          `Failed updating member${err.code.includes("ER_DUP_ENTRY") ? " due to duplicate entry for column" : ""}`,
          err
        );
      });
  }

  /**
   * @description delete member
   * @param {DeleteMemberCommand} deleteMember
   * @returns {Promise<void>}
   */
  static async delete(deleteMember: DeleteMemberCommand): Promise<void> {
    return await dataSource
      .createQueryBuilder()
      .delete()
      .from(member)
      .where("id = :id", { id: deleteMember.id })
      .execute()
      .then(() => {})
      .catch((err) => {
        throw new InternalException("Failed deleting member", err);
      });
  }
}
